package com.akamai.chromecastsample;

import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.v4.view.MenuItemCompat;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.app.MediaRouteActionProvider;
import android.support.v7.media.MediaRouteSelector;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ImageButton;
import android.widget.SeekBar;
import android.widget.TextView;

import com.akamai.chromecast.CastListener;
import com.akamai.chromecast.CastManager;
import com.akamai.chromecast.CastUtils;
import com.akamai.chromecast.SeekBarManager;
import com.akamai.media.IPlayerEventsListener;
import com.akamai.media.VideoPlayerContainer;
import com.akamai.media.VideoPlayerView;
import com.akamai.media.elements.MediaResource;
import com.google.android.gms.cast.CastDevice;
import com.google.android.gms.cast.CastMediaControlIntent;
import com.google.android.gms.cast.MediaStatus;


public class MainActivity extends AppCompatActivity
        implements CastListener, SeekBar.OnSeekBarChangeListener, IPlayerEventsListener, VideoPlayerContainer.VideoPlayerContainerCallback {

    public final static String TAG = "AMPChromecastSample";


    private VideoPlayerContainer mVideoContainer;
    private VideoPlayerView mVideoView;
    private SeekBar mSeekBar;
    private SeekBarManager mSeekBarManager;
    private TextView mSeekBarPosText;
    private ImageButton playpauseButton;
    private CastManager mCastManager;

    private int mCurrentPosition = 0;
    private boolean isComplete = false;
    private String url;
    private MediaResource resource;
    //private Boolean isCasting = false;
    //private boolean mWasPlaying = false;


    protected void onCreate( Bundle savedInstanceState ) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        url = getString(R.string.video_hls);

        mCastManager = new CastManager(getApplicationContext(), CastMediaControlIntent.DEFAULT_MEDIA_RECEIVER_APPLICATION_ID, this);
        mCastManager.addCastListener(this);

        mVideoContainer = (VideoPlayerContainer) findViewById(R.id.playerViewCtrl);
        mVideoContainer.addVideoPlayerContainerCallback(this);
        mVideoContainer.prepareResource(url);

        mSeekBarPosText = (TextView) findViewById(R.id.seekbarText);
        playpauseButton = (ImageButton) findViewById(R.id.playpause);
        mSeekBar = (SeekBar) findViewById(R.id.seekbar);
        mSeekBar.setOnSeekBarChangeListener(this);
    }

    @Override
    public void onVideoPlayerCreated() {
        Log.i(TAG, "onVideoPlayerCreated()");
    }

    @Override
    public void onResourceReady(MediaResource mediaResource) {
        resource = mediaResource;
        mVideoView = mVideoContainer.getVideoPlayer();
        mVideoView.setLicense(getResources().getString(R.string.license));
        mVideoView.setLogEnabled(true);
        mVideoView.addEventsListener(MainActivity.this);
        mVideoView.play(resource);
        mSeekBarManager = new SeekBarManager(mVideoView, mCastManager, mSeekBar, MainActivity.this);
    }

    @Override
    public void onResourceError() {
        Log.e(TAG, "onResourceError()");
    }

    public void togglePlayPause(View view) {
        if (mCastManager.isVideoLoaded()) {
            if (mCastManager.isVideoLoaded() && isComplete) {
                mSeekBarManager.onApplicationStarted();
                isComplete = false;
            } else {
                if (!mCastManager.isPaused()) {
                    mCastManager.pause();
                } else {
                    mCastManager.resume();
                }
            }
        } else {
            if (mVideoView.isFinished() && mVideoView.getStreamUrl() != null) {
                mVideoView.play(resource);
            } else {
                if (mVideoView.isPaused()) {
                    mVideoView.resume();
                } else {
                    mVideoView.pause();
                }
            }
        }
    }

    @Override
    protected void onStart() {
        super.onStart();
        mCastManager.onStart();
    }

    // Remove the selector on stop to tell the media router that it no longer
    // needs to invest effort trying to discover routes of these kinds for now.
    public void onStop() {
        super.onStop();
        mCastManager.onStop();
    }

    public boolean onCreateOptionsMenu( Menu menu ) {
        super.onCreateOptionsMenu(menu);
        getMenuInflater().inflate(R.menu.menu_main, menu);
        MenuItem mediaRouteMenuItem = menu.findItem( R.id.action_cast );
        MediaRouteActionProvider mediaRouteActionProvider = (MediaRouteActionProvider) MenuItemCompat.getActionProvider( mediaRouteMenuItem );
        MediaRouteSelector routeSelector = mCastManager.getMediaRouteSelector();
        mediaRouteActionProvider.setRouteSelector(routeSelector);
        Log.i(TAG, "getMediaRouteSelector: " + routeSelector);
        return true;
    }

    @Override
    protected void onResume() {
        Log.d(TAG, "onResume()");
        if (!mCastManager.isVideoLoaded() && mVideoView!=null) {
            mVideoView.play(resource, mCurrentPosition);
            //mWasPlaying = false;
        }
        super.onResume();
    }

    @Override
    protected void onPause() {
        Log.d(TAG, "onPause()");
        super.onPause();

        if (mVideoView==null)
        {
            return;
        }

        if ((mVideoView.isPlaying() || mVideoView.isPaused()) && !mCastManager.isVideoLoaded()) {
            mCurrentPosition = mVideoView.getTimePosition();
            //mWasPlaying = true;
        }
        mVideoView.stop();
    }

    @Override
    public void onDestroy() {
        Log.d(TAG, "onDestroy()");
        //mVideoView.stop();
        //mVideoView.onDestroy();

        super.onDestroy();
        // mCastManager.onDestroy();
    }

    @Override
    public void onVideoLoaded() {
        Log.i(TAG, "onVideoLoaded()");
    }

    @Override
    public void onApplicationStarted() {
        Log.i(TAG, "onApplicationStarted()");
    }

    @Override
    public void onDeviceSelected(CastDevice device) {
        mVideoView.removeEventsListener(this);
        String deviceName = device.getFriendlyName();
        Log.i(TAG, "onDeviceSelected(): " + deviceName);
    }

    @Override
    public void onDeviceUnselected(long lastPositionMillis) {
        mVideoView.addEventsListener(this);
        mCurrentPosition = (int) (lastPositionMillis / 1000);
        Log.i(TAG, "onDeviceUnselected: lastPosition: " + lastPositionMillis + " - seek: " + mCurrentPosition);
        isComplete = false;
        mVideoView.play(resource, mCurrentPosition);
    }

    @Override
    public void onPlayerStatusChange(int statusCode) {
        switch (statusCode){
            case MediaStatus.IDLE_REASON_FINISHED: setStatus(false);
                isComplete = true;
                break;
            case MediaStatus.PLAYER_STATE_PLAYING: setStatus(true);
                break;
            case MediaStatus.PLAYER_STATE_PAUSED: setStatus(false);
                break;
        }
    }

    @Override
    public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
        int seconds = seekBar.getProgress();
        String secondsStr = CastUtils.getStringFromSeconds(seconds);
        mSeekBarPosText.setText(secondsStr);
        Log.i(TAG, "onProgressChanged: seconds:" + seconds + " mSeekbarPosText.setText: " + secondsStr);
    }

    @Override
    public void onStartTrackingTouch(SeekBar seekBar) {
        Log.i(TAG, "onStartTrackingTouch");

    }

    @Override
    public void onStopTrackingTouch(SeekBar seekBar) {
        Log.i(TAG, "onStopTrackingTouch");
    }

    @Override
    public boolean onKeyDown(int keycode,KeyEvent event){
        if(keycode == KeyEvent.KEYCODE_VOLUME_DOWN){
            mCastManager.setVolume(-1);
            return true;
        }else if(keycode == KeyEvent.KEYCODE_VOLUME_UP){
            mCastManager.setVolume(1);
            return true;
        }
        return super.onKeyDown(keycode,event);
    }

    public void onBackPressed(){
        moveTaskToBack(true);
    }

    public void setStatus(boolean status){
        if(status){
            playpauseButton.setImageResource(R.drawable.pause_icon);
        }else{
            playpauseButton.setImageResource(R.drawable.play_icon);
        }
    }

    public Handler mUIEventsHandler = new Handler(){
        @Override
        public void handleMessage(Message message){
            switch (message.what){
                case PLAYER_EVENT_TYPE_FINISHED:setStatus(false);
                    break;
                case PLAYER_EVENT_TYPE_LOAD_REQUESTED:
                    break;
                case PLAYER_EVENT_TYPE_START_PLAYING: setStatus(true);
                    break;
                case PLAYER_EVENT_TYPE_PAUSE_REQUESTED: setStatus(false);
                    break;
                case PLAYER_EVENT_TYPE_RESUME_REQUESTED: setStatus(true);
                    break;
            }
            super.handleMessage(message);
        }
    };

    @Override
    public boolean onPlayerEvent(int iInEventType) {
        mUIEventsHandler.sendEmptyMessage(iInEventType);
        return true;
    }

    @Override
    public boolean onPlayerExtendedEvent(int i, int i1, int i2) {
        return true;
    }

}
